#!/usr/bin/env python

"""
DOM Level 3 Events support, with SVG Tiny 1.2 implementation additions.
See: http://www.w3.org/TR/DOM-Level-3-Events/events.html

Copyright (C) 2007 Paul Boddie <paul@boddie.org.uk>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
"""

import xml.dom
import time

XML_EVENTS_NAMESPACE = "http://www.w3.org/2001/xml-events"

class EventException(Exception):

    UNSPECIFIED_EVENT_TYPE_ERR = 0
    DISPATCH_REQUEST_ERR = 1

class DocumentEvent:

    """
    An event interface supportable by documents.
    See: http://www.w3.org/TR/DOM-Level-3-Events/events.html#Events-DocumentEvent
    """

    def canDispatch(self, namespaceURI, type):
        return namespaceURI is None and event_types.has_key(type)

    def createEvent(self, eventType):
        try:
            return event_types[eventType]()
        except KeyError:
            raise xml.dom.DOMException(xml.dom.NOT_SUPPORTED_ERR)

class Event:

    "An event class."

    CAPTURING_PHASE = 1
    AT_TARGET = 2
    BUBBLING_PHASE = 3

    def __init__(self):

        "Initialise the event."

        # Initialised later:

        self.target = None
        self.currentTarget = None
        self.defaultPrevented = 0
        self.type = None
        self.namespaceURI = None

        # DOM Level 3 Events:

        self.bubbles = None
        self.eventPhase = self.CAPTURING_PHASE
        self.timeStamp = time.time()

    def initEvent(self, eventTypeArg, canBubbleArg, cancelableArg):
        self.initEventNS(None, eventTypeArg, canBubbleArg, cancelableArg)

    def initEventNS(self, namespaceURIArg, eventTypeArg, canBubbleArg, cancelableArg):
        self.namespaceURI = namespaceURIArg
        self.type = eventTypeArg
        self.bubbles = canBubbleArg
        self.cancelable = cancelableArg

    def preventDefault(self):
        self.defaultPrevented = 1

    def stopPropagation(self):
        pass

    def stopImmediatePropagation(self):
        pass

class UIEvent(Event):

    "A user interface event."

    def __init__(self):
        Event.__init__(self)
        self.detail = None

class MouseEvent(UIEvent):

    "A mouse-related event."

    def __init__(self):
        Event.__init__(self)
        self.screenX, self.screenY, self.clientX, self.clientY, self.button = None, None, None, None, None

# Event types registry.

event_types = {
    "Event" : Event,
    "UIEvent" : UIEvent,
    "MouseEvent" : MouseEvent
    }

class EventTarget:

    "An event target class."

    def addEventListener(self, type, listener, useCapture):
        self.addEventListenerNS(None, type, listener, useCapture)

    def addEventListenerNS(self, namespaceURI, type, listener, useCapture, group=None): # NOTE: group ignored
        listeners = self.ownerDocument.global_.listeners
        if not listeners.has_key(self):
            listeners[self] = {}
        if not listeners[self].has_key((namespaceURI, type)):
            listeners[self][(namespaceURI, type)] = []
        listeners[self][(namespaceURI, type)].append((listener, useCapture))

    def dispatchEvent(self, evt):
        listeners = self.ownerDocument.global_.listeners
        if not evt.type:
            raise EventException(EventException.UNSPECIFIED_EVENT_TYPE_ERR)
        # Dispatch on namespaceURI, type.
        for listener, useCapture in listeners.get(self, {}).get((evt.namespaceURI, evt.type), []):
            listener.handleEvent(evt)
        return evt.defaultPrevented

    def removeEventListener(self, type, listener, useCapture):
        self.removeEventListenerNS(None, type, listener, useCapture)

    def removeEventListenerNS(self, namespaceURI, type, listener, useCapture):
        listeners = self.ownerDocument.global_.listeners
        if listeners.has_key(self) and listeners[self].has_key((namespaceURI, type)):
            try:
                listeners[self][(namespaceURI, type)].remove((listener, useCapture))
            except ValueError:
                pass

# vim: tabstop=4 expandtab shiftwidth=4
